from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

#from copy import deepcopy as dc
from matplotlib.patches import Rectangle
import matplotlib.transforms as transforms

class DataPlotterManager:

    recipe_name = "espda_coadd_spec"

    """ General methods """

    def addSubplots(self, figure):
        self.gs = gridspec.GridSpec(42, 6)
        self.figure = figure
        self.p = Plot()
        self.p.sub_b = self.p.sub(self.figure, self.gs[34:39,0:1], xlabel=None,
                                  ylabel=None)
        self.p.sub_s = self.p.sub(self.figure, self.gs[40:,:], xlabel=None,
                                  ylabel=None)

    def plotProductsGraphics(self):
        self.sel_d = 'flux'
        self.snr_d = False
        self.full_d = False
        self.error_d = False
        self.wavel_range = []
        self.bars = []
        self.plotSpec()


    def plotSpec(self):
        self.plotSpecMain()
        self.plotSpecDetail()


    def readFitsData(self, fitsFiles):
        frames = dict()
        for frame in fitsFiles:
            if frame == '':
                continue
            category = frame.category
            frames[category] = PipelineProduct(frame)

        wavel = frames["RSPEC_PRE"].readTableColumn(1, "WAVEL")
        self.wavel_min = np.min(wavel)
        self.wavel_max = np.max(wavel)
        self.wavel_mean = np.mean(wavel)

        self.rspec_m = Data(frames["RSPEC_PRE"])
        self.rspec_d = Data(frames["RSPEC_PRE"])
        self.rsnr_d = Data(frames["RSPEC_PRE"], y="SNR")
        self.rspec_d.select("WAVEL", (self.wavel_mean-10, self.wavel_mean+10))
        self.rsnr_d.select("WAVEL", (self.wavel_mean-10, self.wavel_mean+10))

        try:
            self.fspec_d = Data(frames["FSPEC_PRE"])
            self.fsnr_d = Data(frames["FSPEC_PRE"], y="SNR")
            self.fspec_d.select("WAVEL",
                                (self.wavel_mean-10, self.wavel_mean+10))
            self.fsnr_d.select("WAVEL",
                               (self.wavel_mean-10, self.wavel_mean+10))
        except:
            self.fspec_d = None
            self.fsnr_d = None

        self.zem = self.rspec_m.frame.readKeyword('HIERARCH ESO DAS Z_EM')


    def setInteractiveParameters(self):
        paramList = list()
        #paramList.append(reflex.RecipeParameter(
        #    recipe=self.recipe_name,
        #    displayName="rebin-flag",
        #    group="Rebinning",
        #    description="Flag to rebin spectra: TRUE, FALSE (disregarded if "
        #        "only SPEC_1D is present)"
        #    ))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "wavel-min",  group = "Rebinning", description =
            "Minimum wavelength [nm]: [300,2500]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "wavel-max",  group = "Rebinning", description =
            "Minimum wavelength [nm]: [300,2500]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "vel-step",   group = "Rebinning", description =
            "Velocity step [km/s]: [0.3, 100.0]"))
        paramList.append(reflex.RecipeParameter(
            recipe=self.recipe_name, displayName="zem", group="Rebinning",
            description="Emission redshift, to be used instead of HIERARCH ESO "
            "OCS OBJ Z EM (in case a better estimation is available)"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName="model-resol", group = "Rebinning", description =
            "Resolution of the instrument model [1.0e3, 1.0e6]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "equal-flag", group = "Equalization", description =
            "Flag to equalize spectra"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "equal-ref",  group = "Equalization", description =
            "ID number of the reference exposure to be used in equalization: " \
            "[0, number of exposures]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "kappa",      group = "Equalization", description =
            "Threshold for kappa-sigma clipping to be used in rebinning: " \
            "[3, 5]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "adp-columns", group = "Input", description =
            "Columns to upload from input in the S1D ADP format (disregarded "
            "if input has a different format) ['WAVE,FLUX,ERR,QUAL']"))

        return paramList


    def setWindowTitle(self):
        title = 'ESPRESSO Data Analysis Software: Spectrum Coaddition'
        return title


    """ Specific methods """

    def plotSpecDetail(self, ret=None):
        tooltip_d = ("Detail of the coadded spectrum (frame category: "
                     "RSPEC_PRE). Use the slider below to shift the central "
                     "wavelength of the plotted region. Check `SNR` to plot "
                     "the SNR spectrum. Check `full` to overplot the "
                     "combination of individual exposures (FSPEC_PRE). Check "
                     "`error` to overplot the error spectrum.")
        self.p.sub_d = self.p.sub(self.figure, self.gs[12:33,:], ret,
                                  tooltip=tooltip_d)
        if self.snr_d:
            self.p.sub_d.set_ylabel(None)
            self.rsnr_d.step(self.p.sub_d, color='g',
                             error=self.error_d)
            if self.full_d:
                self.fsnr_d.scatter(self.p.sub_d, s=1,
                                    color='gainsboro')
        else:
            self.rspec_d.step(self.p.sub_d, error=self.error_d)
            if self.full_d:
                self.fspec_d.scatter(self.p.sub_d, s=20, color='gainsboro')
            #self.p.sub_d.set_ylim(self.ylim_d)
        self.p.sub_d.callbacks.connect('xlim_changed', self.plotSpecRectangle)
        self.p.sub_d.callbacks.connect('ylim_changed', self.plotSpecRectangle)
        self.p.sub_d.legend(prop={'size': 10})
        self.plotSpecRectangle()

    def plotSpecMain(self):
        tooltip_m = ("Coadded spectrum (frame category: RSPEC_PRE). The light "
                     "blue area indicates the wavelength region plotted below.")
        self.p.sub_m = self.p.sub(self.figure, self.gs[:9,:],
                                  title="Coadded spectrum", tooltip=tooltip_m)
        self.rspec_m.step(self.p.sub_m)
        self.ylim_d = (self.p.sub_m.get_ylim()[0],
                       self.p.sub_m.get_ylim()[1])

        trans = transforms.blended_transform_factory(
            self.p.sub_m.transData, self.p.sub_m.transAxes)
        lya_w = (1+self.zem)*121.567
        self.p.sub_m.axvline((1+self.zem)*121.567, color='green')
        self.p.sub_m.text(lya_w, 0, "Lya\nz = %3.2f" % self.zem, ha='center',
                          va='bottom', transform=trans)


    def plotSpecRange(self):
        for r in self.range:
            self.bars.append(self.p.sub_m.axvline(r, color='r'))


    def plotSpecRectangle(self, remove=False):
        try:
            self.rect.remove()
        except:
            pass
        xlim = self.p.sub_d.get_xlim()
        #ylim = self.ylim_d
        ylim = self.p.sub_d.get_ylim()
        (x, y) = (xlim[0], ylim[0])
        w = xlim[1]-xlim[0]
        h = ylim[1]-ylim[0]
        rect = Rectangle((x, y), w, h, edgecolor='lightblue',
                         facecolor='lightblue')
        self.rect = self.p.sub_m.add_patch(rect)


    def plotWidgets(self):
        widgets = list()

        # Wavelength range
        self.clicks = []
        self.range = []
        self.click_m = InteractiveClickableSubplot(self.p.sub_m, self.setRange)
        widgets.append(self.click_m)

        # Check buttons
        if self.fspec_d != None:
            self.toggle = InteractiveCheckButtons(
                self.p.sub_b, self.togglePlotElem, ('SNR', 'full', 'error'),
                (0, 0, 0))
        else:
            self.toggle = InteractiveCheckButtons(
                self.p.sub_b, self.togglePlotElem, ('SNR', 'error'), (0, 0))
        widgets.append(self.toggle)

        # Slider for detail
        self.det = Widget().slider(
            self.p.sub_s, self.refreshDetail, self.wavel_min, self.wavel_max,
            self.wavel_mean, "Wavel.", self.p.sub_m.get_xticks())
        widgets.append(self.det)

        return widgets

    def refreshDetail(self, wavel_sel):
        self.wavel_sel = wavel_sel
        self.rspec_d.select("WAVEL", (self.wavel_sel-10, self.wavel_sel+10))
        self.rsnr_d.select("WAVEL", (self.wavel_sel-10, self.wavel_sel+10))
        try:
            self.fspec_d.select("WAVEL", (self.wavel_sel-10, self.wavel_sel+10))
            self.fsnr_d.select("WAVEL", (self.wavel_sel-10, self.wavel_sel+10))
        except:
            pass
        self.p.sub_d.cla()
        self.plotSpecDetail(ret=self.p.sub_d)


    def setRange(self, point):
        val = self.rspec_m.get_x(point.xdata)

        if val in self.range:
            self.range.remove(val)
        else:
            self.range.append(val)

        self.clicks.append(val)
        if len(np.unique(self.clicks[-3:])) == 1 and len(self.clicks) >= 3:
            self.range = [380, 780]

        if len(self.range) > 2:
            self.range = self.range[-2:]

        for b in self.bars:
            try:
                b.remove()
                self.bars=[]
            except:
                continue

        self.plotSpecRange()

        if len(self.range) == 2:
            new_params = list()
            new_params.append(reflex.RecipeParameter(
                recipe=self.recipe_name, displayName="wavel-min",
                value=str(np.min(self.range))))
            new_params.append(reflex.RecipeParameter(
                recipe=self.recipe_name, displayName="wavel-max",
                value=str(np.max(self.range))))

            return new_params

        """
        self.clicks = self.clicks + 1
        range_limit = self.rspec.getXPoint(point.xdata)
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        new_params = list()
        if self.clicks == 2:
            if len(self.wavel_range) == 2:
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min",
                    value = str(self.wavel_range[0])))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max",
                    value = str(self.wavel_range[1])))
            else:
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min",
                    value = str(380.0)))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max",
                    value = str(780.0)))
        """

    def togglePlotElem(self, sel):
        if sel == 'SNR':
            self.snr_d = not(self.snr_d)
        elif sel == 'full':
            self.full_d = not(self.full_d)
        elif sel == 'error':
            self.error_d = not(self.error_d)
        self.p.sub_d.cla()
        self.plotSpecDetail(ret=self.p.sub_d)


if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
