from __future__ import absolute_import
from __future__ import division
from six.moves import range
from six.moves import zip
try:
    import numpy as np
    import reflex
    import wx
    import wx.grid
    from pipeline_display import *
    from pipeline_product import *
    from reflex_plot_widgets import *
    from numpy.polynomial import Polynomial
    np.seterr(invalid='ignore')
    import_success = True

except ImportError:
    import_success = False

from matplotlib.patches import Rectangle


class Plot:
    def __init__(self):
        pass

    def sub(self, figure, ax, ret=None, title=None,
            xlabel="Wavelength [nm]",
            ylabel= r"Flux density [erg $\mathdefault{cm^2}$ " \
                r"$\mathdefault{s^{-1}}$ $\mathdefault{nm^{-1}}$]",
            tooltip=None, **kwargs):
        if ret is None:
            ret = figure.add_subplot(ax, **kwargs)
        if title != None:
            ret.set_title(title, fontweight='semibold')
        if xlabel != None:
            ret.set_xlabel(xlabel)
        if ylabel != None:
            ret.set_ylabel(ylabel)
        ret.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
        ret.grid(True)
        if tooltip != None:
            ret.tooltip = tooltip
        return ret

    def rectangle(self, event=None, ylim=None, cen=None, remove=True):
        if remove:
            try:
                self.rect.remove()
            except:
                pass
        xlim = self.sub_d.get_xlim()
        if ylim == None:
            ylim = self.sub_d.get_ylim()
        if cen != None:
            xlim = cen/(1-np.array(xlim)/299792.458)
        (x, y) = (xlim[0], ylim[0])
        w = xlim[1]-xlim[0]
        h = ylim[1]-ylim[0]
        rect = Rectangle((x, y), w, h, edgecolor='lightblue',
                         facecolor='lightblue')
        self.rect = self.sub_m.add_patch(rect)

    def rectangles(self, event=None, ylim=None, cen=None):
        try:
            for r in self.rects:
                r.remove()
        except:
            pass
        self.rects = []
        self.sub_m = self.sub_w
        for s, c in zip(self.sub_s, cen):
            self.sub_d = s
            self.rectangle(ylim=ylim, cen=c, remove=False)
            self.rects.append(self.rect)


    def refresh(self, wavel_sel):
        self.wavel_sel = wavel_sel
        self.selectRange(self.wavel_sel)
        self.p.sub_d.cla()
        self.plotSpecDetail()

class Data:
    def __init__(self, frame, x="WAVEL", y="FLUX"):
        self.frame = frame
        self.label = self.frame.readKeyword('HIERARCH ESO PRO CATG')
        self.x = self.frame.readTableColumn(1, x)

        if y == "SNR":
            self.y = self.frame.readTableColumn(1, "FLUX") \
                     / self.frame.readTableColumn(1, "FLUXERR")
            self.label = self.label+", SNR"
        else:
            self.y = self.frame.readTableColumn(1, y)
            self.label = self.label+", flux"

        o_attr = ['e', 'c', 'f', 'n', 'l', 's', 'z']
        o_col = ["FLUXERR", "CONT", "FIT", "NORM", "LINEID", "SYSTID", "LINEZ"]

        for (a, c) in zip(o_attr, o_col):
            try:
                setattr(self, a, self.frame.readTableColumn(1, c))
            except:
                pass

        try:
            self.f = self.f*self.c
            self.r = self.y-self.f
        except:
            pass

        self.sel = list(range(len(self.x)))

    def get_x(self, x_sel, v=1.0):
        dist = np.fabs(self.x*v-x_sel)
        x_id = np.nanargmin(dist)
        return self.x[x_id]


    def get_xy(self, x_sel, y_sel, v=1.0):
        dist = np.fabs(self.x*v-x_sel)
        x_id = np.nanargmin(dist)
        return self.x[x_id], y_sel, y_sel/self.n[x_id]


    def plot(self, subplot, mode, label=None, cen=None, error=False, cont=False,
             fit=False, resid=False, norm=False, **kwargs):

        if cen != None:
            self.v = (1-cen/self.x) * 299792.458 / self.x
        else:
            self.v = np.ones(len(self.x))

        if hasattr(self, 'c') and norm:
            if np.mean(self.c) == -9999:
                self.n = 1/self.y
            else:
                self.n = 1/self.c
        else:
            self.n = np.ones(len(self.x))

        if label == None:
            label = self.label
        ret = getattr(subplot, mode)(self.x[self.sel]*self.v[self.sel],
                                     self.y[self.sel]*self.n[self.sel],
                                     label=label, **kwargs)

        o_attr = ['c', 'f', 'r', 'e']
        o_flag = [cont, fit, resid, error]
        o_label = ["cont.", "fit", "resid.", "error"]
        o_color = ['g', 'lightblue', 'lightblue', 'r']
        for (a, f, l, c) in zip(o_attr, o_flag, o_label, o_color):
            if f:
                try:
                    getattr(subplot, mode)(
                        self.x[self.sel]*self.v[self.sel],
                        getattr(self, a)[self.sel]*self.n[self.sel],
                        label=label+" "+l, color=c, **kwargs)
                except:
                    pass

        return ret


    def scatter(self, subplot, **kwargs):
        return self.plot(subplot, 'scatter', **kwargs)

    def select(self, col, val):
        if hasattr(val, '__len__') and type(val) != str:
            if col is not None:
                self.sel = np.where(np.logical_and(
                    self.frame.readTableColumn(1, col)>=val[0],
                    self.frame.readTableColumn(1, col)<=val[1]))[0]
            else:
                self.sel = np.where(np.logical_and(self.x>=val[0],
                                                   self.x<=val[1]))[0]
        else:
            if val is not None:
                self.sel = np.where(self.frame.readTableColumn(1, col)==val)[0]

    def set_xlim(self, subplot, xlim=None):
        if xlim == None:
            subplot.set_xlim(min(self.x[self.sel]), max(self.x[self.sel]))
        else:
            subplot.set_xlim(xlim)


    def step(self, subplot, xlim=None, **kwargs):
        kwargs['where'] = 'mid'
        ret = self.plot(subplot, 'step', **kwargs)
        self.set_xlim(subplot, xlim)
        return ret

    def line(self, subplot, xlim=None, **kwargs):
        ret = self.plot(subplot, 'plot', **kwargs)
        self.set_xlim(subplot, xlim)
        return ret



class Widget:
    def __init__(self):
        pass

    def slider(self, subplot, callback, valmin, valmax, valinit, label,
               xticks):
        xticks = xticks[np.logical_and(xticks > valmin, xticks < valmax)]
        ret = InteractiveSlider(subplot.axes, callback, valmin, valmax, valinit,
                               label)
        subplot.set_xticks(xticks)
        """
        ret = InteractiveSlider(subplot.axes, callback, valmin, valmax, valinit,
                               label)
        if xticks is not None:
            subplot.set_xticks(xticks)
        """
        return ret

    def slider_discr(self, subplot, callback, valmin, valmax, valinit, label):
        ret = InteractiveSlider(subplot.axes, callback, valmin, valmax, valinit,
                               label)
        subplot.set_xticks(list(range(valmin, valmax)))
        subplot.xaxis.set_ticklabels([])
        for i in range(valmin, valmax):
            subplot.text(i+0.5, 0.5, str(i), ha='center', va='center')
        return ret


class PlotableData:

    def __init__(self, fits, ref_fits=None, mode="plot", x="WAVEL", y="FLUX",
                 x_ref="WAVEL", y_ref="FLUX", e=None, cen=None, color="blue",
                 size=500, width=1.5, marker="+", alpha=1, sel=None, disc=None,
                 ylog=False, decimate=False):
        self.mode = mode
        self.fits = PipelineProduct(fits)
        if x is "VEL":
            self.x = (1 - cen / self.fits.readTableColumn(1, "WAVEL")) *\
                299792.458
            self.cen = cen
        else:
            self.x = self.fits.readTableColumn(1, x)
            self.cen = None
        if decimate == True:
            where = list(range(0, len(self.x), 1000))
            self.x = self.x[where]

        else:
            where = list(range(0, len(self.x)))


        if mode is not "bar":
            if y is "SNR":
                #self.y = self.fits.readTableColumn(1, "FLUX") /\
                #    (self.fits.readTableColumn(1, "FLUXERR") *\
                #    self.fits.readTableColumn(1, "PIXSIZE"))
                self.y = self.fits.readTableColumn(1, "FLUX")[where] /\
                             self.fits.readTableColumn(1, "FLUXERR")[where]
            else:
                self.y = self.fits.readTableColumn(1, y)[where]
        if ylog == True:
            self.x = self.x[self.y!=0]
            self.y = np.log10(self.y[self.y!=0])
        self.e   = None
        self.e_r = None
        if e is not None:
            if e is "RESID":
                self.e = self.fits.readTableColumn(1, "NORM")[where]\
                         -self.fits.readTableColumn(1, "FIT")[where]
                self.e_r = self.fits.readTableColumn(1, "NORMERR")[where]
            else:
                self.e = self.fits.readTableColumn(1, e)[where]

        if sel is not None:
            s   = self.fits.readTableColumn(1, sel[0])[where]
            s_w = np.where((s >= sel[1]) & (s < sel[2]))[0]
            self.x = self.x[s_w]
            if mode is not "bar":
                self.y = self.y[s_w]
            if e is not None:
                self.e = self.e[s_w]
        """
        if decimate == True:
            self.x = self.x[range(0, len(self.x), 1000)]
            self.y = self.y[range(0, len(self.y), 1000)]
        """
        if disc is not None:
            self.x = self.x[disc]
            if mode is not "bar":
                self.y = self.y[disc]
            if e is not None:
                self.e = self.e[disc]

        if ref_fits is not None:
            self.ref_fits = PipelineProduct(ref_fits)
            self.x_ref = self.ref_fits.readTableColumn(1, x_ref)
            self.y_ref = self.ref_fits.readTableColumn(1, y_ref)
            if ylog == True:
                self.y_ref = np.log10(self.y_ref)
        else:
            self.x_ref = self.x
            if mode is not "bar":
                self.y_ref = self.y

        self.color  = color
        self.size   = size
        self.width  = width
        self.marker = marker
        self.alpha  = alpha

    def plot(self, subplot, title=None, tooltip=None, x_label=None,
             y_label=None, x_min=None, x_max=None, y_min=None,
             y_max=None, bars=None, lines=None, line_type='add',mask=None,
             sel=None):
        if x_label is not None:
            subplot.set_xlabel(x_label, style='italic')
        if y_label is not None:
            subplot.set_ylabel(y_label, style='italic')
        if x_min is None:
            x_min = np.nanmin(self.x_ref)
        if x_max is None:
            x_max = np.nanmax(self.x_ref)
        if y_min is None:
            y_min = np.nanmin(self.y_ref)
        if y_max is None:
            y_max = np.nanmax(self.y_ref)
        x_lim = x_min, x_max
        y_lim = y_min, y_max
        if sel is None:
            sel = list(range(len(self.x)))
        if self.mode is "plot":

            subplot.plot(self.x[sel], self.y[sel], color=self.color)
            if self.e is not None:
                subplot.plot(self.x[sel], self.e[sel], color="red")
            if self.e_r is not None:
                subplot.plot(self.x[sel],  self.e_r[sel], color="green")
                subplot.plot(self.x[sel], -self.e_r[sel], color="green")
        if self.mode is "scatter":
            subplot.scatter(self.x[sel], self.y[sel], s=self.size, color=self.color,
                            marker=self.marker, linewidths=self.width,
                            alpha=self.alpha, zorder=10)
        if self.mode is "bar":
            pass
            """
            for x in self.x:
                subplot.axvline(x[sel], color=self.color, linewidth=self.width,
                                alpha=self.alpha)
            """
        subplot.set_xlim(x_lim)
        subplot.set_ylim(y_lim)
        subplot.grid(True)
        if title is not None:
            subplot.set_title(title, fontsize = 12, fontweight = 'semibold')
        if tooltip is not None:
            subplot.tooltip = tooltip
        if bars is not None:
            for bar in bars:
                subplot.axvline(bar, color = "red");
        if lines is not None and len(lines) > 0:
            if line_type == 'add':
                lines_x = np.array(lines[0:len(lines):3])
                lines_y = np.array(lines[1:len(lines):3])
                #if mask is not None:
                if mask != []:
                    try:
                        lines_x = lines_x[mask]
                        lines_y = lines_y[mask]
                    except:
                        pass
                if self.cen is not None:
                    lines_x = (1-self.cen/np.asarray(lines_x))*299792.458
                #for i in range(0, len(lines)):
                subplot.scatter(lines_x, lines_y, s=self.size, color="red",
                                marker=self.marker, linewidths=self.width,
                                alpha=self.alpha, zorder=1)
            elif line_type == 'show':
                for x in lines:
                    subplot.axvline(x, color='red', linewidth=self.width)
    def getXPoint(self, x_sel):
        dist = np.fabs(self.x - x_sel)
        x_id = np.nanargmin(dist)
        return self.x[x_id]

    def getXIndex(self, x_sel):
        dist = np.fabs(self.x - x_sel)
        x_id = np.nanargmin(dist)
        return x_id

    def getPoint(self, x_sel, y_sel):
        dist = np.fabs(self.x - x_sel)
        x_id = np.nanargmin(dist)
        return self.x[x_id], y_sel

class PlotLines:

    def __init__(self, line_fits, x = "WAVEL", y = "PEAK"):
        self.line_fits = PipelineProduct(line_fits)
        self.line_disp = ScatterDisplay()
        self.x = self.line_fits.readTableColumn(1, x)
        self.y = self.line_fits.readTableColumn(1, y)

class PlotArrayRow:
    def __init__(self, line_fits, x = "WAVEL_LOC", y = "FLUX_LOC", z = "FIT_LOC", iline = 235):
        self.line_fits = PipelineProduct(line_fits)
        self.hdulist = self.line_fits.hdulist()
        self.wave_l  = self.line_fits.readTableColumn(1, x)[iline]
        self.spec_l  = self.line_fits.readTableColumn(1, y)[iline]
        self.fit_l   = self.line_fits.readTableColumn(1, z)[iline]
        self.nlines  = len(self.line_fits.readTableColumn(1, x)) - 1


class PlotBars:

    def __init__(self, line_fits, x = "WAVEL", ref = None, comp = None,
            color = "green", style = "solid"):
        self.line_fits = PipelineProduct(line_fits)
        self.x = self.line_fits.readTableColumn(1, x)
        if (ref is not None) and (comp is not None):
            self.ref = self.line_fits.readTableColumn(1, ref)
            self.x = self.x[np.where((self.ref == comp))[0]]
        self.color = color
        self.style = style

    def plot(self, subplot, title = None, tooltip = None, x_label = None,
            y_label = None, x_min = None, x_max = None, y_min = None,
            y_max = None, lines = None):
        for i in range(0, len(self.x)):
            subplot.axvline(self.x[i], color = self.color, linestyle = self.style)


class PlotablePoints:

    def __init__(self, fits, column = "FLUX"):
        self.list      = PipelineProduct(fits)
        self.list_disp = ScatterDisplay()
        self.column    = column
        self.loadFromFits()

    def loadFromFits(self):
        self.list_wavel   = self.list.readTableColumn(1, "WAVEL")
        self.list_column  = self.list.readTableColumn(1, self.column)

    def plot(self, subplot, title, tooltip, color="normal"):
        self.list_disp.setPointSize(100)
        self.list_disp.setColor("Red")
        self.list_disp.display(subplot, title, tooltip, self.list_wavel,
                               self.list_column)

"""
class InteractiveSlider(CallableWidget) :
    def __init__(self, axes, callback, valmin, valmax, valinit, increment, label="") :
        CallableWidget.__init__(self)
        self.inc = increment
        print 'increment=',self.inc
        self.slider = matplotlib.widgets.Slider(axes, label, valmin, valmax, valinit, dragging=True, valfmt='%d')
        sl_xticks = np.arange(valmin,valmax, 10)
        axes.set_xticks(sl_xticks)
        axes.xaxis.set_ticklabels([])
        self.slider.on_changed(self.__callback)
        self.usercallback = callback

    def __callback(self, slider_pos):
        discrete_pos = round(slider_pos / self.inc) * self.inc
        new_params = self.usercallback(discrete_pos)
        self.postCallback(new_params)
"""
