from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy as np
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_synth_spec"
    
    def setInteractiveParameters(self):
        paramList = list()
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-teff", group = "Synth", description =
            "Stellar effective temperature [K]: [3500., 7000]. [5777.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-logg",  group = "Synth", description =
            "Stellar surface gravity log(g[cm/s^2]): [1.0, 5.0]. [4.438]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-feh",      group = "Synth", description =
            "Stellar metallicity [Fe/H]: [-5.0, +0.5]. [0.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-vins",  group = "Synth", description =
            "Instrumental broadening [km/s]: [0., 15.]. [2.606891]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-vrot",  group = "Synth", description =
            "Stellar rotational velocity [km/s]: [0., 300.]. [1.6]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "synth-vmac",  group = "Synth", description =
            "Stellar Macroturbulent velocity [km/s]: [0., 10.]. [3.2]"))
        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        for frame in fitsFiles:
            print(frame)
            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame
        #print self.frames
        #Reading the RV from Keyword:
        self.resf = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("ESO DAS QC SYNTH STDEV RESID W680")

        self.teff = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH TEFF")
        self.logg = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH LOGG")
        self.feh = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH FEH")
        self.vins = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH VINS")
        self.vrot = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH VROT")
        self.vmac = PipelineProduct(self.frames["SMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC SYNTH VMAC")    

        self.smerg_spec = PlotableData(self.frames["SMERG_SPEC"],
                                       y = "FLUX", mode = "plot")
        self.smerg_spec_synt  = PlotableData(self.frames["SMERG_SPEC"],
                                             y = "SYNTH",
                                             ref_fits = self.frames["SMERG_SPEC"],
                                             color = "red", mode = "plot")

        self.smerg_sres = PlotableData(self.frames["SMERG_SPEC"], y = "RESID",
                                       ref_fits = self.frames["SMERG_SPEC"],
                                       color = "green", mode = "plot")
    

        self.smerg_smer = PlotableData(self.frames["SMERG_SPEC"], y = "FLUX",
                                       ref_fits = self.frames["SMERG_SPEC"],
                                       color = "blue",  mode = "plot")

        self.smerg_synt = PlotableData(self.frames["SMERG_SPEC"], y = "SYNTH",
                                       ref_fits = self.frames["SMERG_SPEC"],
                                       color = "red", mode = "plot")

    def addSubplots(self, figure):
        gs_spec = gridspec.GridSpec(1, 1, left=0.15, bottom = 0.70, top = 0.95)
        gs_sres = gridspec.GridSpec(1, 1, left=0.15, bottom = 0.52, top = 0.60)
        gs_smer = gridspec.GridSpec(1, 1, left=0.15, bottom = 0.31, top = 0.42)
        gs_synt = gridspec.GridSpec(1, 1, left=0.15, bottom = 0.10, top = 0.21)
        self.subplot_spec = figure.add_subplot(gs_spec[0,0])
        self.subplot_sres = figure.add_subplot(gs_sres[0,0])
        self.subplot_smer = figure.add_subplot(gs_smer[0,0])
        self.subplot_synt = figure.add_subplot(gs_synt[0,0])


    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        spec_title   = "Normalized observed and synthetic spectra"
        spec_tooltip = "Blue: rebinned normalized spectrum; red: synthetic spectrum.\n"
        spec_y_label = r"Normalized Flux"
        
        sres_title   = "Residuals Observed - Synthetic spectra"
        sres_tooltip = "Green: residuals: observed-synthetic fluxes \n"
        sres_y_label = r"Flux residuals"
        
        smer_title   = "Normalized observed spectrum"
        smer_tooltip = "Blue: rebinned normalized spectrum \n"
        smer_y_label = r"Normalized Flux"
        
        synt_title   = "Synthetic spectrum"
        synt_tooltip = "Red: synthetic spectrum \n"
        synt_y_label = r"Normalized Flux"

        all_x_label  = "Wavelength [nm]"
        
        self.fits = PipelineProduct(self.frames["SMERG_SPEC"])
        y = self.fits.readTableColumn(1, "RESID")
        #print len(y)
        ymax = np.nanmax(y)
        ymin = np.nanmin(y)
        dy = (ymax - ymin) / 10.
        #print dy, ymax, ymin
        
        self.smerg_spec.plot(self.subplot_spec, title = spec_title,
                             tooltip = spec_tooltip, x_label = all_x_label,
                             y_min=-0.1,y_max=1.2,
                             y_label = spec_y_label, bars = self.wavel_range)
        

        self.smerg_spec_synt.plot(self.subplot_spec, title = spec_title,
                                  tooltip = spec_tooltip, x_label = all_x_label,
                                  y_min=-0.1,y_max=1.2,
                                  y_label = spec_y_label)

        self.smerg_sres.plot(self.subplot_sres, title = sres_title,
                             tooltip = sres_tooltip, x_label = all_x_label,
                             #y_min=ymin-dy,y_max=ymax+dy,
                             y_min=-1.-dy,y_max=1+dy,
                             y_label = sres_y_label, bars = self.wavel_range)
    
        self.smerg_smer.plot(self.subplot_smer, title = smer_title,
                         tooltip = smer_tooltip, x_label = all_x_label,
                         y_min=-0.2,y_max=1.2,
                         y_label = smer_y_label, bars = self.wavel_range)
        
        self.smerg_synt.plot(self.subplot_synt, title = synt_title,
                         tooltip = synt_tooltip, x_label = all_x_label,
                         y_min=-0.2,y_max=1.2,
                         y_label = synt_y_label, bars = self.wavel_range)

        #Text plot
        self.subplot_sres.text(0.95, 0.75, r"RMS($\lambda$<680nm) = " +
                               str(np.around(self.resf, decimals=3)),
                               horizontalalignment="right",
                               verticalalignment="bottom",
                               fontsize=8, transform=self.subplot_sres.transAxes)

        self.subplot_synt.text(0.95, 0.05, "Teff = " +
                               str(np.around(self.teff)) + "  Logg = " +
                               str(np.around(self.logg, decimals=2)) +
                               "  [Fe/H]= " +
                               str(np.around(self.feh, decimals=2)) +
                               "  Vins= " +
                               str(np.around(self.vins, decimals=2)) +
                               "  Vrot= " +
                               str(np.around(self.vrot, decimals=1)) +
                               "  Vmac= " +
                               str(np.around(self.vmac, decimals=1)),
                               horizontalalignment="right",
                               verticalalignment="bottom", fontsize=8,
                               transform=self.subplot_synt.transAxes)

    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_spec = InteractiveClickableSubplot(self.subplot_spec, 
            self.wavelRange)
        widgets.append(self.clickable_spec)
        return widgets

    def wavelRange(self, point):
        self.clicks = self.clicks + 1
        range_limit = self.smerg_spec.getXPoint(point.xdata)        
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        new_params = list()

        return new_params

    def setWindowTitle(self):
      title = 'ESPRESSO Data Analysis Software: Synthetic Spectrum Creation' 
      return title

if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
